# frozen_string_literal: true

module HyraxHelper
  include ::BlacklightHelper
  include Hyrax::BlacklightOverride
  include Hyrax::HyraxHelperBehavior

  def available_translations
    {
      'de' => 'Deutsch',
      'en' => 'English',
      'es' => 'Español',
      'fr' => 'Français',
      'it' => 'Italiano',
      'pt-BR' => 'Português do Brasil',
      'zh' => '中文'
    }.slice(RDMS::Application.config.i18n.available_locales )
  end

  def s3_file_download_url_for_file_set(file_set_id)
    s3 = S3StorageService.new
    s3.init_client

    file_set = FileSet.find(file_set_id)
    bucket_name = s3.sanitise_name(file_set.parent_works.first.id)

    s3.presigned_url_for_downlad(bucket_name, file_set.object_key_on_s3)
  end

  def can_create_work_in_group?(group_id)
    current_user.admin? || current_user.crc_manager_for?(group_id) || current_user.group_manager_for?(group_id) || current_user.group_member_for?(group_id)
  end

  def can_manage_managers_for_group?(presenter)
    current_user.admin? || current_user.crc_manager_for?(presenter.id)
  end

  def can_manage_members_for_group?(presenter)
    current_user.admin? || current_user.crc_manager_for?(presenter.id) || current_user.group_manager_for?(presenter.id)
  end

  def manager_role(presenter)
    Collection.find(presenter.id).manager_role
  end

  def member_role(presenter)
    Collection.find(presenter.id).member_role
  end

  def manager_role_for_group(presenter)
    Collection.find(presenter.id).manager_role_for_group
  end

  def member_role_for_group(presenter)
    Collection.find(presenter.id).member_role_for_group
  end

  def within_file_size_threshold?(presenter)
    work = ActiveFedora::Base.find(presenter.id)
    total_size = work.total_file_size
    total_size > 0 && total_size < ENV.fetch('DOWNLOAD_SIZE_LIMIT', '100000000').to_i
  end

  def is_tombstonable?(presenter)
    presenter.tombstone_status != ['initiated'] && (is_published?(presenter) || is_archived?(presenter))
  end

  def is_published?(presenter)
    ['deposited', 'published'].include?(presenter.workflow.state)
  end

  def is_archived?(presenter)
    presenter.workflow.state == 'archived'
  end

  def tombstone_out_of_date?(presenter)
    ((Date.today) - presenter.tombstone_date.first.to_date).to_i >= ENV['NUMBER_OF_DAYS_TO_TOMBSTONE'].to_i
  end

  def is_tombston_initiated?(presenter)
    presenter.tombstone_status.present? && presenter.tombstone_status == ['initiated']
  end

  def is_permanently_tombstoned?(presenter)
    presenter.tombstone_status == ['deleted'] && presenter.is_tombstoned
  end
end
