# frozen_string_literal: true

module ComplexField
  module IdentifierIndexer
    def generate_solr_document
      super.tap do |solr_doc|
        index_identifier(solr_doc)
      end
    end

    def index_identifier(solr_doc)
      solr_doc[Solrizer.solr_name('complex_identifier', :symbol)] = object.complex_identifier.map do |i|
        i.identifier.reject(&:blank?).first
      end
      solr_doc[Solrizer.solr_name('complex_identifier', :displayable)] = object.complex_identifier.to_json
      facetable_ids = %w[group_id project_id]

      object.complex_identifier.each do |i|
        next if i.scheme.all?(&:blank?) || i.identifier.all?(&:blank?)

        label = i.scheme.reject(&:blank?).first.downcase.tr(' ', '_')
        fld_name = Solrizer.solr_name("complex_identifier_#{label}", :symbol)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << i.identifier.reject(&:blank?)
        solr_doc[fld_name].flatten!
        next unless facetable_ids.include?(label)

        fld_name = Solrizer.solr_name("complex_identifier_#{label}", :facetable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << i.identifier.reject(&:blank?)
        solr_doc[fld_name].flatten!
      end
    end

    def self.identifier_facet_fields
      # solr fields that will be used for a search
      fields = []
      fields << Solrizer.solr_name('complex_identifier_group_id', :facetable)
      fields << Solrizer.solr_name('complex_identifier_project_id', :facetable)
      fields
    end

    def self.identifier_search_fields
      # solr fields that will be used for a search
      fields = []
      fields << Solrizer.solr_name('complex_identifier', :symbol)
      fields
    end

    def self.identifier_show_fields
      # solr fields that will be used to display results on the record page
      fields = []
      fields << Solrizer.solr_name('complex_identifier', :displayable)
      fields
    end
  end
end
