require "./lib/vocabularies/rdms"
# frozen_string_literal: true
# Generated by hyrax:models:install
class FileSet < ActiveFedora::Base
  self.indexer = CrcFileSetIndexer

  after_save :create_file_set_with_fast_load_meta_data
  after_destroy :destroy_set_meta_data

  property :for_complex_identifier, predicate: ::RDF::Vocab::Rdms.forComplexIdentifier, multiple: false  do |index|
    index.as :stored_searchable, :facetable
  end

  property :for_complex_type, predicate: ::RDF::Vocab::Rdms.forComplexType, multiple: false  do |index|
    index.as :stored_searchable, :facetable
  end
  include ::Hyrax::FileSetBehavior

  def fast_load_meta_data
    FileSetFastLoadMetadata.find_by(fedora_file_set_id: id)
  end

  def mime_type_image?
    file_extention = ".#{self.title.first&.split('.')&.last}"
    mime_type = Rack::Mime.mime_type(file_extention)
    mime_type.include?('image')
  end

  def object_key_on_s3
    prefix = case for_complex_type
    when 'ComplexSubject'
       complex_subject = ComplexSubject.find_by(source_identifier: for_complex_identifier)
      "#{complex_subject.subject_title}/files"
    when 'ComplexSession'
      complex_session = ComplexSession.find_by(source_identifier: for_complex_identifier)
      "#{complex_session.complex_subject.subject_title}/#{complex_session.session_title}/files"
    when 'ComplexModality'
      complex_modality = ComplexModality.find_by(source_identifier: for_complex_identifier)
      "#{complex_modality.complex_subject.subject_title}/#{complex_modality.complex_session.session_title}/#{complex_modality.modality_title}/files"
    else
      'files'
    end

    "/#{prefix}/#{title.first}"
  end

  private

  def create_file_set_with_fast_load_meta_data
    work_source_identifier = self.for_complex_identifier || self.parent_works.first&.source&.first
    work_type = self.for_complex_type || "CrcDataset"
    
    metadata = FileSetFastLoadMetadata.find_or_initialize_by(fedora_file_set_id: self.id)
    metadata.work_source_identifier = work_source_identifier
    metadata.work_model_name = work_type
    metadata.file_set_title = self.title&.first || self.label
    metadata.save
  end

  def destroy_set_meta_data
    meta_data = FileSetFastLoadMetadata.find_by(fedora_file_set_id: self.id)
    meta_data.destroy! if meta_data.present?
  end
end
